"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.reqMemberInfo = exports.reqAttention = exports.openQbQuoteWindow = exports.openQbCalculateWindow = exports.openPage = exports.openNewBondDetail = exports.modifyBondAttention = exports.inQbBrowser = exports.inQb = exports.getUser = exports.getSystemTime = exports.getQbVersion = exports.getLanguage = exports.getChangeBondResult = exports.getBondAttention = exports.api = void 0;

var _jsBase = _interopRequireDefault(require("js-base64"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var Base64 = _jsBase["default"].Base64;
var cefQuery = window.cefQuery;
var qb = null;

var inQb = function inQb() {
  return !!window.cefQuery;
};

exports.inQb = inQb;

var inQbBrowser = function inQbBrowser() {
  return !!window.QBbrowser;
};

exports.inQbBrowser = inQbBrowser;

if (inQbBrowser()) {
  qb = new window.QBbrowser();
}

var api = function api(funcName, reqStr, success, failure) {
  if (!inQb()) {
    return;
  }

  cefQuery({
    request: reqStr,
    onSuccess: success || function (response) {
      console.info(response);
    },
    onFailure: failure || function (errorCode, errorMessage) {
      console.info(funcName + ' : ' + errorMessage);
    }
  });
}; // 获取QB登录用户信息


exports.api = api;

var getUser = function getUser(success, failure) {
  var reqStr = '["req_cache",[{"data":"UserInfo"}]]';
  api('getUser', reqStr, function (res) {
    var user = JSON.parse(res);
    success({
      id: user.UserId,
      username: user.UserAccount,
      password: user.Password
    });
  }, failure);
}; // 打开QB精简报价


exports.getUser = getUser;

var openQbQuoteWindow = function openQbQuoteWindow(bondInfo, success, failure) {
  var reqStr = "[\"open_page\", [{\"name\":\"bond_short_quote\",\"bondkey\":\"".concat(bondInfo.bondkey, "\",\"listmarket\":\"").concat(bondInfo.listedmarket, "\"}]]");
  api('openQbQuoteWindow', reqStr, function (res) {
    var result = JSON.parse(res);
    success(result);
  }, failure);
}; // 打开QB价格试算


exports.openQbQuoteWindow = openQbQuoteWindow;

var openQbCalculateWindow = function openQbCalculateWindow(bondInfo, success, failure) {
  var reqStr = "[\"open_page\", [{\"name\":\"bondcalculate\",\"bondkey\":\"".concat(bondInfo.bondkey, "\",\"listmarket\":\"").concat(bondInfo.listedmarket, "\"}]]");
  api('openQbCalculateWindow', reqStr, function (res) {
    var result = JSON.parse(res);
    success(result);
  }, failure);
}; // 查询QB版本


exports.openQbCalculateWindow = openQbCalculateWindow;

var getQbVersion = function getQbVersion(success, failure) {
  var reqStr = "[\"req_cache\", [{\"data\":\"Version\"}]]";
  api('getQbVersion', reqStr, function (res) {
    var result = JSON.parse(res);
    success({
      version: result.version
    });
  }, function (error_code, error_message) {
    failure({
      errorCode: error_code,
      errorMessage: error_message
    });
  });
}; // 查询是否可切换至指定债券


exports.getQbVersion = getQbVersion;

var getChangeBondResult = function getChangeBondResult(bondInfo, success, failure) {
  var reqStr = "[\"change_bond\", [{\"bondkey\":\"".concat(bondInfo.bondkey, ".").concat(bondInfo.listedmarket, "\"}]]");
  api('getChangeBondResult', reqStr, function (res) {
    var result = JSON.parse(res);
    success({
      changed: result.changed
    });
  }, function (error_code, error_message) {
    failure({
      errorCode: error_code,
      errorMessage: error_message
    });
  });
}; // 打开新的债券详情tab页


exports.getChangeBondResult = getChangeBondResult;

var openNewBondDetail = function openNewBondDetail(bondInfo, success, failure) {
  var reqStr = "[\"open_page\", [{\"name\":\"bond_detail\",\"bondkey\":\"".concat(bondInfo.bondkey, "\",\"listmarket\":\"").concat(bondInfo.listedmarket, "\"}]]");
  api('openQbQuoteWindow', reqStr, function (res) {
    var result = JSON.parse(res);
    console.log('调用打开新tab接口：', result);
  }, failure);
}; // 查询当前债券是否存在关注组


exports.openNewBondDetail = openNewBondDetail;

var getBondAttention = function getBondAttention(bondInfo, success, failure) {
  var reqStr = "[\"req_cache\", [{\"data\":\"BondAttention\",\"callback\":\"handleManageAttention\",\"BondKey\":\"".concat(bondInfo.bondkey, "\",\"ListedMarket\":\"").concat(bondInfo.listedmarket, "\",\"Todo\":\"Check\"}]]");
  api('getBondAttention', reqStr, function (res) {
    var result = JSON.parse(res);
    success(result);
  }, function (error_code, error_message) {
    failure({
      errorCode: error_code,
      errorMessage: error_message
    });
  });
}; // 管理当前债券关注组


exports.getBondAttention = getBondAttention;

var modifyBondAttention = function modifyBondAttention(bondInfo, success, failure) {
  var reqStr = "[\"req_cache\", [{\"data\":\"BondAttention\",\"callback\":\"handleManageAttention\",\"BondKey\":\"".concat(bondInfo.bondkey, "\",\"ListedMarket\":\"").concat(bondInfo.listedmarket, "\",\"Todo\":\"Modify\"}]]");
  api('modifyBondAttention', reqStr, function (res) {
    var result = JSON.parse(res);
    success(result);
  }, function (error_code, error_message) {
    failure({
      errorCode: error_code,
      errorMessage: error_message
    });
  });
}; // 获取我的关注组


exports.modifyBondAttention = modifyBondAttention;

var reqAttention = function reqAttention(success, failure) {
  if (qb) {
    qb.getCommon(['tds_req', {
      type: 'tds.req.system.attention'
    }, ''], function (res) {
      console.log(res);
    }, function (error_code, error_message) {
      console.log(error_code, error_message);
    });
  } else {
    window.tds_reqAttention = function (result) {
      var _ref = result || {},
          group = _ref.group,
          finish = _ref.finish;

      typeof success === 'function' && success(group || [], finish);
    };

    var reqStr = "[\"tds_req\", [{\"req\": \"".concat(Base64.encode(JSON.stringify({
      type: 'tds.req.system.attention'
    })), "\", \"callback\": \"").concat(Base64.encode('tds_reqAttention'), "\"}]]");
    api('reqAttention', reqStr, function (res) {
      console.log(res);
    }, function (code, message) {
      console.log(code, message);
    });
  }
};

exports.reqAttention = reqAttention;

var getSystemTime = function getSystemTime(success, failure) {
  window.tds_getSystemTime = function (result) {
    var _ref2 = result || {},
        time = _ref2.time;

    typeof success === 'function' && success(time);
  };

  var reqStr = "[\"tds_req\", [{\"req\": \"eyJ0eXBlIjoidGRzLnJlcS5zeXN0ZW0udGltZSJ9\", \"callback\": \"".concat(Base64.encode('tds_getSystemTime'), "\"}]]");
  api('getSystemTime', reqStr, function (res) {
    console.log(res);
  }, function (code, message) {
    console.log(code, message);
  });
};

exports.getSystemTime = getSystemTime;

var reqMemberInfo = function reqMemberInfo(callback, timeout) {
  var timer;

  function clear() {
    if (timer) {
      clearTimeout(timer);
      timer = null;
    }
  }

  window.tds_reqMemberInfo = function (result) {
    clear();
    typeof callback === 'function' && callback(result);
  };

  if (timeout) {
    timer = setTimeout(function () {
      clear();
      typeof callback === 'function' && callback();
    }, timeout);
  }

  var type = 'tds.req.system.user.member.info';
  var reqStr = "[\"tds_req\", [{\"req\": \"".concat(Base64.encode(JSON.stringify({
    type: type
  })), "\", \"callback\": \"").concat(Base64.encode('tds_reqMemberInfo'), "\"}]]");
  api('reqMemberInfo', reqStr, function (res) {
    console.log('reqMemberInfo : ', res);
  }, function (code, message) {
    console.log('reqMemberInfo : ', code, message);
  });
};

exports.reqMemberInfo = reqMemberInfo;

var openPage = function openPage(pageInfo) {
  pageInfo = pageInfo || {};

  if (!pageInfo.name) {
    return;
  }

  var req = ['open_page', [pageInfo]];
  api('open_page', JSON.stringify(req), function (res) {
    console.log("open qb page ".concat(pageInfo.name), res);
  });
};

exports.openPage = openPage;

var getLanguage = function getLanguage(success, failure) {
  if (!inQb()) {
    typeof failure === 'function' && failure();
    return;
  }

  window.tds_getLanguage = function (result) {
    var _ref3 = result || {},
        language = _ref3.language;

    typeof success === 'function' && success(language);
  };

  var reqStr = "[\"tds_req\", [{\"req\": \"eyJ0eXBlIjoidGRzLnJlcS5zeXN0ZW0ubGFuZ3VhZ2UuaW5mbyJ9\", \"callback\": \"".concat(Base64.encode('tds_getLanguage'), "\"}]]");
  api('getLanguage', reqStr, function (res) {
    console.log(res);
  }, function (code, message) {
    console.log(code, message);
    typeof failure === 'function' && failure();
  });
};

exports.getLanguage = getLanguage;