"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.initialReducerGroup = initialReducerGroup;
exports.reducerBuilder = reducerBuilder;

var _redux = require("redux");

var _invariant = _interopRequireDefault(require("invariant"));

var _createReducer = _interopRequireDefault(require("./createReducer"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toArray(arr) { return _arrayWithHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableRest(); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e2) { throw _e2; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e3) { didErr = true; err = _e3; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var __rest = void 0 && (void 0).__rest || function (s, e) {
  var t = {};

  for (var p in s) {
    if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0) t[p] = s[p];
  }

  if (s != null && typeof Object.getOwnPropertySymbols === "function") for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
    if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i])) t[p[i]] = s[p[i]];
  }
  return t;
};

function reducerBuilder(options, onReducer) {
  var reducers = {};
  var reducerGroups = new Map();

  for (var key in options) {
    if (options.hasOwnProperty(key)) {
      var reducer = options[key];

      if (reducer instanceof Array) {
        collectReducers(reducerGroups, reducer);
      } else {
        reducers[key] = reducer;
      }
    }
  }

  var _iterator = _createForOfIteratorHelper(reducerGroups.entries()),
      _step;

  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var _step$value = _slicedToArray(_step.value, 2),
          _key = _step$value[0],
          reducerGroup = _step$value[1];

      if (reducers.hasOwnProperty(_key)) {
        throw Error('Duplicate key ' + _key);
      }

      reducers[_key] = initialReducerGroup(reducerGroup, onReducer);
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }

  return (0, _redux.combineReducers)(reducers);
}

function collectReducers(reducerGroups, reducers) {
  reducers.forEach(function (reducer) {
    var keys = reducer.key.split('.');

    var _keys = _toArray(keys),
        groupKey = _keys[0],
        subKeys = _keys.slice(1);

    var group = reducerGroups.get(groupKey);

    if (!group) {
      group = new Map();
      reducerGroups.set(groupKey, group);
    }

    if (subKeys.length === 0) {
      reducer.single = true;
    } else {
      reducer.subKeys = subKeys;
    }

    if (group.has(reducer.key) && !reducer.single) {
      throw Error('Duplicate key ' + reducer.key);
    }

    group.set(reducer.key, reducer);
  });
}

function initialReducerGroup(reducerGroup, onReducer) {
  var handlers = {};
  var initialState = {};

  var _iterator2 = _createForOfIteratorHelper(reducerGroup.values()),
      _step2;

  try {
    var _loop = function _loop() {
      var reducer = _step2.value;

      if (!reducer.resultKey) {
        reducer.resultKey = 'result';
      }

      if (reducer.single) {
        initialState = reducer.initialState || {};
      } else {
        overrideState(initialState, reducer.subKeys, reducer.initialState);
      }

      var reducerAction = reducer.action;

      if (!reducerAction) {
        reducerAction = reducer.key;
      }

      (0, _invariant["default"])(reducerAction, "[reducer.key] reducer key is not empty");
      handlers[reducerAction] = reducerHandler(reducer, 'loading', function (state, action) {
        var _Object$assign;

        var payload = action.payload,
            other = __rest(action, ["payload"]);

        delete other.type;
        var newState = Object.assign((_Object$assign = {}, _defineProperty(_Object$assign, reducer.resultKey, null), _defineProperty(_Object$assign, "payload", payload), _defineProperty(_Object$assign, "success", false), _defineProperty(_Object$assign, "loading", true), _defineProperty(_Object$assign, "fromSocket", false), _Object$assign), other);

        if (typeof onReducer === 'function') {
          newState = onReducer(newState, state, action, 'loading');
        }

        return newState;
      });
      handlers["".concat(reducerAction, "_SUCCESS")] = reducerHandler(reducer, 'success', function (state, action) {
        var done = action.done,
            fromSocket = action.fromSocket,
            result = action.result,
            payload = action.payload,
            other = __rest(action, ["done", "fromSocket", "result", "payload"]);

        delete other.type;
        var newState = {};

        if (fromSocket && done) {
          newState = result;
        } else {
          var _Object$assign2;

          newState = Object.assign((_Object$assign2 = {}, _defineProperty(_Object$assign2, reducer.resultKey, result), _defineProperty(_Object$assign2, "payload", payload), _defineProperty(_Object$assign2, "success", true), _defineProperty(_Object$assign2, "loading", false), _defineProperty(_Object$assign2, "fromSocket", fromSocket), _Object$assign2), other);
        }

        if (typeof onReducer === 'function') {
          newState = onReducer(newState, state, action, 'success');
        }

        return newState;
      });
      handlers["".concat(reducerAction, "_FAIL")] = reducerHandler(reducer, 'fail', function (state, action) {
        var payload = action.payload,
            error = action.error,
            other = __rest(action, ["payload", "error"]);

        delete other.type;
        var newState = Object.assign({
          payload: payload,
          error: error,
          success: false,
          loading: false,
          fromSocket: false
        }, other);

        if (typeof onReducer === 'function') {
          newState = onReducer(newState, state, action, 'fail');
        }

        return newState;
      });
    };

    for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
      _loop();
    }
  } catch (err) {
    _iterator2.e(err);
  } finally {
    _iterator2.f();
  }

  return (0, _createReducer["default"])(initialState, handlers);
}

function overrideState(state, keys) {
  var value = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var length = keys.length;

  if (length === 1) {
    return state[keys[0]] = value;
  }

  var previous = state;

  for (var i = 0; i < length; ++i) {
    if (i === length - 1) {
      previous[keys[i]] = value;
    } else {
      var next = previous[keys[i]];

      if (!next) {
        next = previous[keys[i]] = {};
      }

      previous = next;
    }
  }

  return;
}

function reducerHandler(reducer, method, handler) {
  return function (state, action) {
    var result;

    if (reducer[method]) {
      result = reducer[method](state, action);
    } else {
      result = handler(state, action);
    }

    if (reducer.single) {
      state = result;
    } else {
      state = Object.assign({}, state);
      overrideState(state, reducer.subKeys, result);
    }

    return state;
  };
}