import ReactDOM from 'react-dom';
import { createBrowserHistory, createHashHistory } from 'history';
import invariant from 'invariant';
import { patchHistory, isFunction, isHTMLElement } from './utils';
import { HistoryType } from './types';
import create from './redux';
import Logger from './logger';
import App from './App';
const defaultOpts = {
    historyType: HistoryType.BROWSER
};
export default function (opts = defaultOpts) {
    const { onEffect, onFetchOption, onReducer, historyType } = opts;
    let history;
    switch (historyType) {
        case HistoryType.HASH:
            history = createHashHistory();
            break;
        case HistoryType.BROWSER:
            history = createBrowserHistory();
            break;
        default:
            history = createBrowserHistory();
            break;
    }
    const options = {
        setupApp(app) {
            app.history = patchHistory(history);
            if (window.__INITIAL_WORKER_PATH__) {
                try {
                    const path = window.__INITIAL_WORKER_PATH__;
                    Logger.start(path);
                }
                catch (e) {
                    console.error('parse window initial logger path error -> ', e);
                }
            }
        },
        onEffect,
        onFetchOption,
        onReducer,
        history
    };
    const app = create(options);
    const appStart = app.rStart;
    app.router = router;
    app.start = start;
    return app;
    function render(container, store, app) {
        ReactDOM.render(App(app), container);
    }
    function start(container) {
        if (typeof container === 'string') {
            container = document.querySelector(container);
            invariant(container, `[app.start] container ${container} not found`);
        }
        invariant(!container || isHTMLElement(container), `[app.start] container should be HTMLElement`);
        invariant(app._router, `[app.router] router must be registered before app.start()`);
        if (!app.store) {
            appStart(app);
        }
        const store = app.store;
        if (container) {
            return render(container, store, app);
        }
        else {
            return App(app);
        }
    }
    function router(router) {
        invariant(isFunction(router), `[app.router] router should be function, but got ${typeof router}`);
        app._router = router;
    }
}
