class Logger {
    // ------------------------- methods ------------------------------
    static checkWork() {
        return !!this.worker;
    }
    static getTime(type) {
        let timeStamp;
        switch (type) {
            case 'millisecond':
                timeStamp = new Date().getTime();
                break;
            case 'second':
                timeStamp = Date.parse(new Date()) / 1000;
                break;
            default:
                timeStamp = Date.parse(new Date()) / 1000;
        }
        return timeStamp;
    }
    // --------------------------- worker ---------------------------------
    static start(path) {
        if (!window.SharedWorker) {
            console.error('Check if Browser supports the Worker api.');
        }
        if (!path) {
            console.error('please provide the url of worker.js');
            return;
        }
        this.worker = new SharedWorker(path);
    }
    static info(params, options) {
        if (!this.checkWork()) {
            console.error('please run Logger.start() first');
            return;
        }
        if (params.hasOwnProperty('eventTime')) {
            this.worker.port.postMessage(params);
        }
        else {
            this.worker.port.postMessage(Object.assign(Object.assign({}, params), { eventTime: this.getTime(options === null || options === void 0 ? void 0 : options.timeStamp) }));
        }
    }
    static onMessage(func) {
        if (!this.checkWork()) {
            console.error('please run Logger.start() first');
            return;
        }
        this.worker.port.onmessage = function (e) {
            func && func(e);
        };
    }
}
export { Logger };
export default Logger;
