import createSagaMiddleware from 'redux-saga';
import { Base64 } from 'js-base64';
import createStore from './createStore';
import { reducerBuilder } from './reducerBuilder';
import { sagaBuilder } from './sagaBuilder';
const cSagaMiddleware = createSagaMiddleware;
export const _models_ = Symbol('_models_');
export default function create(createOpts) {
    const { setupApp, 
    // onError: onErr,
    onEffect, onFetchOption, onReducer, history } = createOpts;
    const app = {
        [_models_]: [],
        model,
        models,
        rStart
    };
    function model(model) {
        app[_models_] = [...app[_models_], model];
    }
    function models(models) {
        app[_models_] = [...app[_models_], ...models];
    }
    function rStart(app) {
        const sagaMiddleware = cSagaMiddleware();
        let initialState = {};
        if (window.__INITIAL_STATE__) {
            try {
                initialState =
                    JSON.parse(Base64.decode(window.__INITIAL_STATE__)) || {};
            }
            catch (e) {
                console.error('parse window initial state error -> ', e);
            }
        }
        app.store = createStore({
            reducers: reducerBuilder(app[_models_], onReducer),
            initialState,
            sagaMiddleware
        });
        app.store.runSaga = sagaMiddleware.run;
        const sagas = sagaBuilder(app[_models_], {
            onEffect,
            onFetchOption,
            history
        });
        sagaMiddleware.run(sagas);
        setupApp(app);
    }
    return app;
}
